<?php
namespace Teknisa\SUP\Services;

use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use \Zeedhi\Framework\DB\StoredProcedure\Param;

/**
 * SupplyTek Approval/Disapproval Model
 * 
 * Connects to the Database to handle queries 
 * related to the approval/dusapproval workflow. 
 *  
 * @author Rômulo Alves Lousada
 */
class SupplyTekApprovalWorkflowService
{
    protected $connection;
    protected $newCodeSC;
    
    /**
     * Class constructor
     * 
     * Initialize an object's properties upon creation of the object.
     * 
     * @param object $entityManager Object to access the database
     * @param object $newCodeSC Object to generate new codes without commit
     * @param object $environment Object to access environment session data
     * 
     * @return void
     * 
     * @access public
     * @author Rômulo Alves Lousada
     */
    public function __construct($entityManager, $newCodeSC, $environment)
    {
        $this->entityManager = $entityManager;
        $this->connection = $entityManager->getConnection();
        $this->newCodeSC = $newCodeSC;
        $this->environment = $environment;
    }
    
    /**************************************************************************/
    /*                                UTILITIES                               */
    /**************************************************************************/
    
    /**
     * Start a transaction on database
     * 
     * @return void
     * 
     * @access public
     * @author Rômulo Alves Lousada
     */
    public function beginTransaction()
    {
        $this->entityManager->beginTransaction();
    }
    
    /**
     * Commit a transaction on database
     * 
     * @return void
     * 
     * @access public
     * @author Rômulo Alves Lousada
     */
    public function commitTransaction()
    {
        $this->entityManager->commit();
    }
    
    /**
     * Rollback a transaction on database
     * 
     * @return void
     * 
     * @access public
     * @author Rômulo Alves Lousada
     */
    public function rollbackTransaction()
    {
        $this->entityManager->rollBack();
    }
    
    /**
     * Generate New Code without Commit
     * 
     * Generates and retrieves the new code to the table sent, needed to save
     * as a PK from the table.
     * 
     * @param string $tableName String containing the table name to generate the code
     *
     * @return string
     * 
     * @access private
     * @author Rômulo Alves Lousada
     */
    private function generateCodeSC($tableName)
    {
        return $this->newCodeSC->getCodeByTableName($tableName, $this->environment->getNrOrg());
    }
    
    /**
     * Set environment values
     * 
     * @param array $row Array with orders information
     *
     * @return void
     * 
     * @access public
     * @author Danilo D. Vaz
     */
    public function setEnvironment($row)
    {
        $this->environment->setNrOrg($row['NRORG']);
        $this->environment->setNrOrgTrab($row['NRORG']);
        $this->environment->setCdFilial($row['CDFILIAL']);
        $this->environment->setCdOperador($row['CDOPERADOR']);
    }
    
    
    /**************************************************************************/
    /*                             WORKFLOW START                             */
    /**************************************************************************/
    
    /**
     * Update status to Approved or Disapproved
     * 
     * Receives the status from the request and updates the table
     * accordingly, the starting function of the workflow process.
     * 
     * @param array $params Array with request data received to make the process
     *
     * @return void
     * @throws Exception
     * 
     * @access public
     * @author Rômulo Alves Lousada
     */
    public function saveStatus($params)
    {
        try {
            $sql = "UPDATE 
                        APROVALCADA
                    SET 
                        IDAPROVADO = DECODE(:P_STATUS, 'A', 'APROVADO', 'RECUSADO'), 
                        DSJUSTIFICA = :P_DSJUSTIFIC,
                        DTAPROVACAO = SYSDATE, 
                        CDOPERAPROV = :P_CDOPERADOR
                    WHERE 
                        NRAPROVALCADA = :P_NRAPROVALCADA";
            
            $std = $this->connection->prepare($sql);
            
            $std->execute(
                array(
                    "P_STATUS" => $params["STATUS"],
                    "P_DSJUSTIFIC" => $params["DSJUSTIFIC"],
                    "P_CDOPERADOR" => $params["CDOPERADOR"],
                    "P_NRAPROVALCADA" => $params["NRAPROVALCADA"]
                )
            );
        } catch(\Exception $e) {

            throw $e;
        }
    }
    
    /**************************************************************************/
    /*                          DISAPPROVAL WORKFLOW                          */
    /**************************************************************************/
    
    /**
     * Update AUTOFORNE status to Disapproved
     * 
     * @param array $params Array with request data received to make the process
     *
     * @return void
     * @throws Exception
     * 
     * @access public
     * @author Rômulo Alves Lousada
     */
    public function updateAFDisapproval($params)
    {
        try {
            $sql = "UPDATE 
                        AUTOFORNE
                    SET 
                        IDBAIXAAF = DECODE(:P_STATUS, 'A', NULL, 5),
                        IDSITAFALCADA = DECODE(:P_STATUS, 'A', 'APROVADA', 'RECUSADA'), 
                        DTAPROVALCADA = SYSDATE
                    WHERE 
                        CDFILDEST = :P_CDFILDEST 
                        AND CDFILLANC = :P_CDFILLANC 
                        AND NRAUTOFORN = :P_NRAUTOFORN";
            
            $std = $this->connection->prepare($sql);
            
            $std->execute(
                array(
                    "P_STATUS" => $params["STATUS"],
                    "P_CDFILDEST" => $params["CDFILDEST"],
                    "P_CDFILLANC" => $params["CDFILLANC"],
                    "P_NRAUTOFORN" => $params["NRAUTOFORN"]
                )
            );
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * Return required AF data on disapproval process
     * 
     * Searches the database and retrieves AF data needed
     * on the disapproval workflow
     * 
     * @param array $params Array with request data received to make the process
     *
     * @return array
     * @throws Exception
     * 
     * @access public
     * @author Rômulo Alves Lousada
     */
    public function getAFDataDisapproval($params)
    {
        try {
            $sql = "SELECT 
                        AUTOFORNE.DTAUTOFORN, 
                        AUTOFORNE.CDOPERLANCAF, 
                        OPERADOR.NMOPERADOR AS NMDESTINATARIO, 
                        OPERADOR.DSEMAILOPER AS EMAILDEST
                    FROM 
                        AUTOFORNE, 
                        OPERADOR
                    WHERE 
                        AUTOFORNE.CDFILDEST = :P_CDFILDEST 
                        AND AUTOFORNE.CDFILLANC = :P_CDFILLANC
                        AND AUTOFORNE.NRAUTOFORN = :P_NRAUTOFORN
                        AND AUTOFORNE.CDOPERLANCAF = OPERADOR.CDOPERADOR";
            
            $std = $this->connection->prepare($sql);
            
            $std->execute(
                array(
                    "P_CDFILDEST" => $params["CDFILDEST"],
                    "P_CDFILLANC" => $params["CDFILLANC"],
                    "P_NRAUTOFORN" => $params["NRAUTOFORN"]
                )
            );
            
            return $std->fetchAll();
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * Return required User data on disapproval process
     * 
     * Searches the database and retrieves User data needed
     * on the disapproval workflow 
     * 
     * @param array $params Array with request data received to make the process
     *
     * @return array
     * @throws Exception
     * 
     * @access public
     * @author Rômulo Alves Lousada
     */
    public function getUserDataDisapproval($params)
    {
        try {
            $sql = "SELECT 
                        NMOPERADOR AS NMREPROV, 
                        DSEMAILOPER
                    FROM 
                        OPERADOR
                    WHERE 
                        CDOPERADOR = :P_CDOPERADOR";
            
            $std = $this->connection->prepare($sql);
            
            $std->execute(
                array(
                    "P_CDOPERADOR" => $params["CDOPERADOR"]
                )
            );
            
            return $std->fetchAll();
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * Insert into NOTIFICSUPPLY a new notification
     * 
     * @param array $params Array with request data received to make the process
     * @param array $data Array with data needed on the process
     *
     * @return void
     * @throws Exception
     * 
     * @access public
     * @author Rômulo Alves Lousada
     */
    public function saveNotificationDisapproval($params, $data)
    {
        try {
            $nmNotificacao = "Reprovação da AF - Nº " . $params["NRAUTOFORN"];
            $dsTextNotific = "O seu pedido infelizmente foi reprovado por " . $data["USER"][0]["NMREPROV"] . " dia " . date("d/m/Y") . ". Motivo da reprovação: " . $params["DSJUSTIFIC"];
            
            $sql = "INSERT INTO NOTIFICSUPPLY 
                    (
                        NRNOTIFICSUPPLY, 
                        CDFILIAL, 
                        CDOPERADOR, 
                        NMNOTIFICACAO, 
                        DSTEXTNOTIFIC, 
                        IDNOTIFICLIDO, 
                        NRAUTOFORN, 
                        DTINCLUSAO, 
                        CDOPERINCLUSAO
                    )
                    VALUES 
                    (
                        :P_NRNOTIFICSUPPLY, 
                        :P_CDFILDEST, 
                        :P_CDOPERLANCAF, 
                        :P_NMNOTIFICACAO, 
                        :P_DSTEXTNOTIFIC, 
                        'N', 
                        :P_NRAUTOFORN, 
                        SYSDATE, 
                        :P_CDOPERADOR
                    )";
            
            $std = $this->connection->prepare($sql);

            $std->execute(
                array(
                    "P_NRNOTIFICSUPPLY" => (string) $this->generateCodeSC("NOTIFICSUPPLY"),
                    "P_CDFILDEST" => $params["CDFILDEST"],
                    "P_CDOPERLANCAF" => $data["AF"][0]["CDOPERLANCAF"],
                    "P_NMNOTIFICACAO" => $nmNotificacao,
                    "P_DSTEXTNOTIFIC" => $dsTextNotific,
                    "P_NRAUTOFORN" => $params["NRAUTOFORN"],
                    "P_CDOPERADOR" => $params["CDOPERADOR"]
                )
            );
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    /**************************************************************************/
    /*                            APPROVAL WORKFLOW                           */
    /**************************************************************************/
    
    /**
     * Return NRCONFIGALCADA from APROVALCADA
     * 
     * @param array $params Array with request data received to make the process
     *
     * @return string
     * @throws Exception
     * 
     * @access public
     * @author Rômulo Alves Lousada
     */
    public function getNrConfigAlcada($params)
    {
        try {
            $sql = "SELECT 
                        NRCONFIGALCADA
                    FROM 
                        APROVALCADA
                    WHERE 
                        NRAPROVALCADA = :P_NRAPROVALCADA";
            
            $std = $this->connection->prepare($sql);
            
            $std->execute(
                array(
                    "P_NRAPROVALCADA" => $params["NRAPROVALCADA"]
                )
            );
            
            return $std->fetchAll()[0]["NRCONFIGALCADA"];
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * Return required data to send next approver email
     * 
     * Searches the database and retrieves User data needed
     * on the email workflow 
     * 
     * @param array $params Array with request data received to make the process
     *
     * @return array
     * @throws Exception
     * 
     * @access public
     * @author Rômulo Alves Lousada
     */
    public function getDataApproval($params)
    {
        try {
            $sql = "SELECT 
                        AUTOFORNE.CDFILDEST, 
                        FILIAL.NMFILIAL AS NMFILDEST, 
                        AUTOFORNE.DTAUTOFORN, 
                        (
                            SELECT 
                                MAX(DTPROGENTAF)
                            FROM 
                                ITEMAUTF
                            WHERE 
                                ITEMAUTF.CDFILDEST = AUTOFORNE.CDFILDEST 
                                AND ITEMAUTF.CDFILLANC = AUTOFORNE.CDFILLANC
                                AND ITEMAUTF.NRAUTOFORN = AUTOFORNE.NRAUTOFORN
                        ) AS DTPROGENTAF,
                        AUTOFORNE.VRTOTALAF, 
                        COMPRADOR.CDOPERADOR || ' - ' || COMPRADOR.NMOPERADOR AS COMPRADOR,
                        AUTOFORNE.CDOPERLANCAF, 
                        OPERADOR.NMOPERADOR AS NMSOLICITANTE, 
                        OPERADOR.DSEMAILOPER AS EMAILSOLICITANTE,
                        AUTOFORNE.CDFORNECED, 
                        FORMATMASK(FORNECEDOR.IDTPIJURFORN, FORNECEDOR.NRINSJURFORN) AS NRINSJURFORN, 
                        FORNECEDOR.NMFANTFORN,
                        EMAILFORNE.DSEMAILFORN, 
                        PARAMGERALECOMMERCE.DSLINKECOMMERCE
                    FROM 
                        AUTOFORNE, 
                        OPERADOR, 
                        FILIAL, 
                        FORNECEDOR,
                        OPERADOR COMPRADOR,
                        PARAMGERALECOMMERCE,
                        TABLE
                        (
                            DADOS_FORN_EMAIL
                            (
                                FORNECEDOR.CDFORNECED, 'S', '', AUTOFORNE.NRORG 
                            )
                        ) EMAILFORNE
                    WHERE 
                        AUTOFORNE.CDFILDEST = :P_CDFILDEST 
                        AND AUTOFORNE.CDFILLANC = :P_CDFILLANC
                        AND AUTOFORNE.NRAUTOFORN = :P_NRAUTOFORN
                        AND AUTOFORNE.CDOPERLANCAF = OPERADOR.CDOPERADOR
                        AND AUTOFORNE.CDFILDEST = FILIAL.CDFILIAL
                        AND AUTOFORNE.CDFORNECED = FORNECEDOR.CDFORNECED
                        AND AUTOFORNE.CDOPERADOR = COMPRADOR.CDOPERADOR
                        AND FILIAL.CDFILCENTCMP = PARAMGERALECOMMERCE.CDFILIAL (+)";
            
            $std = $this->connection->prepare($sql);
            
            $std->execute(
                array(
                    "P_CDFILDEST" => $params["CDFILDEST"],
                    "P_CDFILLANC" => $params["CDFILLANC"],
                    "P_NRAUTOFORN" => $params["NRAUTOFORN"]
                )
            );
            
            return $std->fetchAll();
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * Return required AF data to send email
     * 
     * Searches the database and retrieves AF data needed
     * on the email workflow
     * 
     * @param array $params Array with request data received to make the process
     *
     * @return array
     * @throws Exception
     * 
     * @access public
     * @author Rômulo Alves Lousada
     */
    public function searchForNextApprover($params)
    {
        try {
            $sql = "SELECT 
                        ALCADAGERFILI.CDOPERADOR AS CDPROXAPROV, 
                        VRAPROVCMP, 
                        OPERADOR.NMOPERADOR AS NMPROXAPROV, 
                        OPERADOR.DSEMAILOPER AS EMAILPROXAPROV
                    FROM 
                        ALCADAGERFILI, 
                        OPERADOR
                    WHERE 
                        ALCADAGERFILI.NRCONFIGALCADA = :P_NRCONFIGALCADA
                        AND ALCADAGERFILI.CDFILIAL = :P_CDFILDEST
                        AND ALCADAGERFILI.CDOPERADOR = OPERADOR.CDOPERADOR
                        AND VRAPROVCMP = 
                        (
                            SELECT 
                                MIN(VRAPROVCMP)
                            FROM 
                                ALCADAGERFILI
                            WHERE 
                                NRCONFIGALCADA = :P_NRCONFIGALCADA
                                AND CDFILIAL = :P_CDFILDEST
                                AND CDOPERADOR <> :P_CDOPERADOR
                                AND VRAPROVCMP > :P_VRAPROVCMP
                        )";
            
            $std = $this->connection->prepare($sql);
            
            $std->execute(
                array(
                    "P_NRCONFIGALCADA" => $params["NRCONFIGALCADA"],
                    "P_CDFILDEST" => $params["CDFILDEST"],
                    "P_CDOPERADOR" => $params["CDOPERADOR"],
                    "P_VRAPROVCMP" => (float)$params["VRAPROVCMP"]
                )
            );
            
            return $std->fetchAll();
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    /*************************************/
    /*         HAVE NEXT APPROVER        */
    /*************************************/
    
    /**
     * Insert into NOTIFICSUPPLY a new notification
     * 
     * @param array $params Array with request data received to make the process
     * @param array $approver Array with next approver data required to save on the database
     *
     * @return void
     * @throws Exception
     * 
     * @access public
     * @author Rômulo Alves Lousada
     */
    public function saveNextApproval($params, $approver)
    {
        try {
            $sql = "INSERT INTO APROVALCADA 
                    (
                        NRAPROVALCADA, 
                        NRCONFIGALCADA, 
                        CDFILDEST, 
                        CDFILLANC, 
                        NRDEMANDA,
                        CDOPERADOR, 
                        IDTPDEMANDA, 
                        IDCLASSIFIC, 
                        IDOPERACAOALC, 
                        IDAPROVADO, 
                        NRORG, 
                        DTINCLUSAO
                    )
                    VALUES 
                    (
                        :P_APROVALCADA, 
                        :P_NRCONFIGALCADA, 
                        :P_CDFILDEST, 
                        :P_CDFILLANC, 
                        :P_NRAUTOFORN, 
                        :P_CDPROXAPROV, 
                        'AF', 
                        'PADRAO', 
                        'LANCAMENTO', 
                        'PENDENTE', 
                        :P_NRORG, 
                        SYSDATE
                    )";
            
            $std = $this->connection->prepare($sql);
            
            $std->execute(
                array(
                    "P_APROVALCADA" => (int) $this->generateCodeSC("APROVALCADA"),
                    "P_NRCONFIGALCADA" => $params["NRCONFIGALCADA"],
                    "P_CDFILDEST" => $params["CDFILDEST"],
                    "P_CDFILLANC" => $params["CDFILLANC"],
                    "P_NRAUTOFORN" => $params["NRAUTOFORN"],
                    "P_CDPROXAPROV" => $approver["CDPROXAPROV"],
                    "P_NRORG" => $this->environment->getNrOrg()
                )
            );
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    /*************************************/
    /*      DONT HAVE NEXT APPROVER      */
    /*************************************/
    
    /**
     * Delete itens from ITEMAUTF
     * 
     * If the item inside the order has 0 quantity to be delivered, means
     * it has been removed from the order, so it must be deleted from the table.
     * 
     * @param array $params Array with request data received to make the process
     *
     * @return void
     * @throws Exception
     * 
     * @access public
     * @author Rômulo Alves Lousada
     */
    public function removeItensWith0Quantity($params)
    {
        try {
            $sql = "DELETE FROM 
                        ITEMAUTF
                    WHERE 
                        CDFILDEST = :P_CDFILDEST
                        AND CDFILLANC = :P_CDFILLANC
                        AND NRAUTOFORN = :P_NRAUTOFORN
                        AND QTPROGENTAF = 0";
                        
            $std = $this->connection->prepare($sql);

            $std->execute(
                array(
                    "P_CDFILDEST" => $params["CDFILDEST"],
                    "P_CDFILLANC" => $params["CDFILLANC"],
                    "P_NRAUTOFORN" => $params["NRAUTOFORN"]
                )
            );
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * Inserts data in ENVIOAFFORN
     * 
     * @param array $params Array with request data received to make the process
     * @param array $dataAF Array with data needed to finish process.
     *
     * @return string
     * @throws Exception
     * 
     * @access public
     * @author Rômulo Alves Lousada
     */
    public function saveSentAFApproval($params, $dataAF)
    {
        try {
            $nrSeqEnvAF = (string) $this->generateCodeSC("ENVIOAFFORN");
            
            $sql = "INSERT INTO ENVIOAFFORN 
                    (
                        NRSEQENVIOAFORN, 
                        CDFORNECED, 
                        CDPAIS, 
                        SGESTADO, 
                        DTINIAPURAFFORN, 
                        DTFINAPURAFFORN, 
                        DTENVIOAFFORN, 
                        QTAFSENVIOFOR, 
                        QTAFSNENVFOR, 
                        IDTIPENVLIDO, 
                        IDTIPENVAFFORN, 
                        CDOPERINCLUSAO, 
                        DTENTREGAFFORN, 
                        IDTCLASSENVAFOR
                    )
                    SELECT 
                        :P_NRSEQENVIOAFORN, 
                        F.CDFORNECED, 
                        E.CDPAIS, 
                        E.SGESTADO, 
                        :P_DTAUTOFORN, 
                        :P_DTAUTOFORN, 
                        SYSDATE, 
                        1, 
                        0, 
                        'N', 
                        NVL(F.IDTPENVAFNORMAL, 'E'), 
                        :P_CDOPERADOR, 
                        :P_DTPROGENTAF, 
                        'N'
                    FROM 
                        FORNECEDOR F, 
                        ENDEFORN E
                    WHERE 
                        F.CDFORNECED = :P_CDFORNECED
                        AND F.CDFORNECED = E.CDFORNECED
                        AND E.IDTPENDEFORN = 'P'";
            
            $std = $this->connection->prepare($sql);

            $std->execute(
                array(
                    "P_NRSEQENVIOAFORN" => $nrSeqEnvAF,
                    "P_DTAUTOFORN" => $dataAF["DTAUTOFORN"],
                    "P_CDOPERADOR" => $params["CDOPERADOR"],
                    "P_CDFORNECED" => $dataAF["CDFORNECED"],
                    "P_DTPROGENTAF" => $dataAF["DTPROGENTAF"]
                )
            );
            
            return $nrSeqEnvAF;
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * Updates data in AUTOFORNE
     * 
     * @param array $params Array with request data received to make the process
     * @param int $nrSeqEnvAF Sequential number generated when adding new register to ENVIOAFFORN.
     *
     * @return void
     * @throws Exception
     * 
     * @access public
     * @author Rômulo Alves Lousada
     */
    public function updateAFApproval($params, $nrSeqEnvAF)
    {
        try {
            $sql = "UPDATE 
                        AUTOFORNE
                    SET 
                        NRSEQENVIOAFORN = :P_NRSEQENVIOAFORN,
                        IDBAIXAAF = DECODE(:P_STATUS, 'A', NULL, 5),
                        IDSITAFALCADA = DECODE(:P_STATUS, 'A', 'APROVADA', 'RECUSADA'), 
                        DTAPROVALCADA = SYSDATE
                    WHERE 
                        CDFILDEST = :P_CDFILDEST
                        AND CDFILLANC = :P_CDFILLANC
                        AND NRAUTOFORN = :P_NRAUTOFORN";
            
            $std = $this->connection->prepare($sql);
            
            $std->execute(
                array(
                    "P_NRSEQENVIOAFORN" => $nrSeqEnvAF,
                    "P_STATUS" => $params["STATUS"],
                    "P_CDFILDEST" => $params["CDFILDEST"],
                    "P_CDFILLANC" => $params["CDFILLANC"],
                    "P_NRAUTOFORN" => $params["NRAUTOFORN"]
                )
            );
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * Updates data in AUTOFORNE
     * 
     * @param array $params Array with request data received to make the process
     * @param array $dataAF Array with data needed to finish process.
     *
     * @return void
     * @throws Exception
     * 
     * @access public
     * @author Rômulo Alves Lousada
     */
    public function saveNotificationApproval($params, $dataAF)
    {
        try {
            $nmNotification = "Aprovação da AF - Nº " . $params["NRAUTOFORN"];
            
            $sql = "INSERT INTO NOTIFICSUPPLY 
                    (
                        NRNOTIFICSUPPLY, 
                        CDFILIAL, 
                        CDOPERADOR, 
                        NMNOTIFICACAO, 
                        DSTEXTNOTIFIC, 
                        IDNOTIFICLIDO, 
                        NRAUTOFORN, 
                        DTINCLUSAO, 
                        CDOPERINCLUSAO
                    )
                    VALUES 
                    (
                        :P_NRNOTIFICSUPPLY, 
                        :P_CDFILDEST, 
                        :P_CDOPERLANCAF,
                        :P_NMNOTIFICACAO,
                        'O seu pedido foi aprovado por todos os gestores e foi enviado para o fornecedor',
                        'N', 
                        :P_NRAUTOFORN, 
                        SYSDATE, 
                        :P_CDOPERADOR
                    )";
            
            $std = $this->connection->prepare($sql);
            
            $std->execute(
                array(
                    "P_NRNOTIFICSUPPLY" => (string) $this->generateCodeSC("NOTIFICSUPPLY"),
                    "P_CDFILDEST" => $params["CDFILDEST"],
                    "P_CDOPERLANCAF" => $dataAF["CDOPERLANCAF"],
                    "P_NMNOTIFICACAO" => $nmNotification,
                    "P_NRAUTOFORN" => $params["NRAUTOFORN"],
                    "P_CDOPERADOR" => $params["CDOPERADOR"]
                )
            );
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * @param $params supplier CDFORNECED parameter
     * @return IDTPENVAFNORMAL suplier parameter
     * @throws Exception
     * 
     * @access public
     * @author Tarcisio Junior
     */
    public function getSupplierAFParameter($params) 
    {
        try {
            $sql = "SELECT IDTPENVAFNORMAL FROM FORNECEDOR WHERE CDFORNECED = :P_CDFORNECED";
            
            $std = $this->connection->prepare($sql);
            
            $std->execute(
                array(
                    "P_CDFORNECED" => $params
                )
            );
            
            return $std->fetchAll();
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * @param $params supplier CDFORNECED parameter
     * @return DSEMAILCOT suplier email
     * @throws Exception
     * 
     * @access public
     * @author Tarcisio Junior
     */
    public function getSupplierEmail($params) 
    {
        try {
            $sql = "SELECT DSEMAILCOT FROM FORNECEDOR WHERE CDFORNECED = :P_CDFORNECED";
            
            $std = $this->connection->prepare($sql);
            
            $std->execute(
                array(
                    "P_CDFORNECED" => $params
                )
            );
            
            return $std->fetchAll();
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * @return array report parameters
     * 
     * @access public
     * @author Tarcisio Junior
     */
    
    public function getReportParameters($params) 
    {
        return array (
            '__report_name'   => 'padraoEnvioAF',
            'CDFILDEST'       => $params["CDFILDEST"],
            'NRAUTOFORN'      => $params["NRAUTOFORN"],
            'CDFILLANCPADAF'  => $params["CDFILLANC"],
            'IDEXIBEOBSFIL'   => 'S',
            'IMPRIMEOBSPROD'  => 'S',
            "NRORG"           => $this->environment->getNrOrg()
        );
    }
    
    /**
     * @return IDSISDOS Parameter
     * @throws Exception
     * 
     * @access public
     * @author Tarcisio Junior
     */
    
    public function getIdSisdosParameter() 
    {
        try {
            $sql = "SELECT IDSISDOS FROM PARAMGERAL";
            
            $std = $this->connection->prepare($sql);
            
            $std->execute();
            
            return $std->fetchAll();
        } catch(\Exception $e) {
            throw $e;
        }
    }
}